@@ -1,15 +1,18 @@
 #!/usr/bin/perl -w
-# $Id: whatsnewfm.pl,v 1.96 2006/09/13 17:30:57 mastermitch Exp $
+# $Id: whatsnewfm.pl,v 1.112 2009/03/26 20:55:14 mastermitch Exp $
 #############################################################################
 #
-my $id="whatsnewfm.pl  v0.6.6  2006-09-13";
+my $id='whatsnewfm.pl  v0.7.0  2009-03-26';
 #   Filters the freshmeat newsletter for 'new' or 'interesting' entries.
 #   
-#   Copyright (C) 2000-2006  Christian Garbs <mitch@cgarbs.de>
+#   Copyright (C) 2000-2009  Christian Garbs <mitch@cgarbs.de>
 #                            Joerg Plate <Joerg@Plate.cx>
 #                            Dominik Brettnacher <dominik@brettnacher.org>
 #                            Pedro Melo Cunha <melo@isp.novis.pt>
 #                            Matthew Gabeler-Lee <msg2@po.cwru.edu>
+#                            Bernd Rilling <brilling@ifsw.uni-stuttgart.de>
+#                            Jost Krieger <Jost.Krieger@ruhr-uni-bochum.de>
+#
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -27,6 +30,10 @@ my $id="whatsnewfm.pl  v0.6.6  2006-09-13";
 #
 #############################################################################
 #
+# v0.7.0
+# 2009/03/23--> Add scoring of licenses via LICSCORE.
+# 2009/03/19--> BUGFIX: Support the new Newsletter format of FM3
+#
 # v0.6.6
 # v0.6.5
 # 2004/04/11--> Make score regexp match multiline as proposed in
@@ -197,7 +204,7 @@ my $id="whatsnewfm.pl  v0.6.6  2006-09-13";
 # 2000/07/06--> first piece of code
 #
 #
-# $Id: whatsnewfm.pl,v 1.96 2006/09/13 17:30:57 mastermitch Exp $
+# $Id: whatsnewfm.pl,v 1.112 2009/03/26 20:55:14 mastermitch Exp $
 #
 #
 #############################################################################
@@ -448,6 +455,14 @@ sub do_scoring($)
 	    }
 	}
     }
+
+    if (defined $app->{'license'}) {
+	foreach my $score ( @{$config->{'LICSCORE'}}) {
+	    if ($app->{'license'} =~ /$score->{'regexp'}/is) {
+		$app->{'score'} += $score->{'score'};
+	    }
+	}
+    }
 }
 
 
@@ -548,10 +563,8 @@ sub parse_newsletter()
 
     my $subject      = "Subject: Freshmeat Newsletter (no subject?)\n";
     my $encoding     = '';
-    my $position     = 3;
-    # 3-> after mail header
-    # 2-> within articles
-    # 1-> after articles
+    my $position     = 1;
+    # 1-> after mail header
     # 0-> within releases
 
     my $hot_written  = 0;
@@ -559,10 +572,11 @@ sub parse_newsletter()
     my $db_expired   = 0;
     my $db_new       = 0;
 
-    my $articles     = 0;
     my $releases     = 0;
     my $releases_new = 0;
 
+    my $end = 0;
+
     my $hot_applications = [];
     my $new_applications = [];
 
@@ -570,7 +584,6 @@ sub parse_newsletter()
 
 ### generate current timestamp
 
-
     my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
 	localtime(time);
 
@@ -585,13 +598,11 @@ sub parse_newsletter()
 
 ### read databases
 
-
     $database    = open_old_db();
     $interesting = open_hot_db();
 
 
 ### expire 'old' entries
-
     
     foreach my $number (keys %{$database}) {
 	if (($database->{$number}+$config->{'EXPIRE'}) < $timestamp_cmp) {
@@ -602,8 +613,7 @@ sub parse_newsletter()
 
 
 ### process email headers
-
-
+ 
     while (<STDIN>) {
 	last if /^$/;
 	if (/^Subject:\s/) {
@@ -620,108 +630,20 @@ sub parse_newsletter()
 
 ### process email body
 
-
     while (my $line=<STDIN>) {
 	chomp $line;
-	if (($position == 3) and ($line =~ /^::: A R T I C L E S \(\d+\) :::/)) {
-	    $position = 2;
-	} elsif (($position > 0) and ($line =~ /^::: R E L E A S E   D E T A I L S :::/)) {
+	if (($position > 0) and ($line =~ /^SOFTWARE:/)) {
 	    $position = 0;
-	} elsif ($position == 2) {
-	    
-### parse an article entry
-
-	    # empty line
-	    while ((defined $line) and ($line =~ /^\s*$/)) {
-		$line=<STDIN>;
-	    }
-	    next unless defined $line;
-
-	    # title
-	    chomp $line;
-	    $new_app->{'subject'} = $line;
-	    $line=<STDIN>;
-	    next unless defined $line;
-	    
-	    # from
-	    if ($line =~ /^by /) {
-		$line =~ s/^by //;
-		chomp $line;
-		$new_app->{'author'} = $line;
-		$line=<STDIN>;
-		next unless defined $line;
-	    }
-
-	    # section
-	    if ($line =~ /^Section: /) {
-		$line =~ s/^Section: //;
-		chomp $line;
-		$new_app->{'category'} = $line;
-		$line=<STDIN>;
-		next unless defined $line;
-	    }
-
-	    # date
-	    chomp $line;
-	    $new_app->{'date'} = $line;
-	    $line=<STDIN>;
-	    next unless defined $line;
-	    
-	    # empty line
-	    while ((defined $line) and ($line =~ /^\s*$/)) {
-		$line=<STDIN>;
-	    }
-	    next unless defined $line;
-
-	    # text
-	    $new_app->{'description'} = $line;
-	    while ($line=<STDIN>) {
-		last if $line =~ /^\s$/;
-		$new_app->{'description'} .= $line;
-	    }
-
-	    # empty line
-	    while ((defined $line) and ($line =~ /^\s*$/)) {
-		$line=<STDIN>;
-	    }
-	    next unless defined $line;
-
-	    # URL
-	    if ($line =~ /^\s*URL: /) {
-		$line =~ s/^\s*URL: //;
-		chomp $line;
-		$new_app->{'newslink'} = $line;
-		$line=<STDIN>;
-		next unless defined $line;
-	    }
-
-	    # save it
-	    undef $new_app->{'project_id'};
-	    $articles++;
-	    
-	    do_scoring($new_app);
-
-	    push @{$new_applications}, $new_app;
-	    $new_app = {};
-
-	    # wait for separator  (UGLY, change this routine somehow)
-	    my $end = 0;
 	    while ($end == 0) {
 		chomp $line;
-		if ($line =~ /- % -/) {
-		    $end = 1;
-		} else {
-		    if ($line !~ /^\s*$/) {
-			if (($line =~ tr/. -//c) == 0) {
-			    $position = 1;
-			    $end = 1;
-			}		    
-		    }
+		if ($line !~ /^\s*$/) {
+		    if ($line =~ /========================================================================/) {
+			$end = 1;
+		    }	
 		}
 		$line=<STDIN>;
 		$end = 1 unless defined $line;
 	    }
-
 	} elsif ($position == 0) {
 	    
 ### parse an release entry
@@ -735,11 +657,11 @@ sub parse_newsletter()
 	    my $release_nr;
 
 	    # title
-	    while ($line !~ /^\[\d+\] - /) {
+	    while ((defined $line) and ($line !~ /^\[\d+\] /)) {
 		$line=<STDIN>;
-		next unless defined $line;
 	    }
-	    $line =~ s/^(\[\d+\]) - //;
+	    next unless defined $line;
+	    $line =~ s/^(\[\d+\]) //;
 	    $release_nr = $1;
 	    chomp $line;
 	    $new_app->{'subject'} = $line;
@@ -752,58 +674,15 @@ sub parse_newsletter()
 	    }
 	    next unless defined $line;
 	    
-	    # from
-	    if ($line =~ /^\s+by /) {
-		$line =~ s/^\s+by //;
-		chomp $line;
-		$new_app->{'author'} = $line;
-		$line=<STDIN>;
-		next unless defined $line;
-	    }
-
-	    # date
-	    chomp $line;
-	    while ((defined $line) && ($line =~ /\(http:\/\/.*\)/ )) {
-		# This is no date, but a multi-line "by" field!
-		$new_app->{'author'} .= "\n              $line";
-		$line=<STDIN>;
-		chomp $line;
-	    }
-	    next unless defined $line;
-	    $line =~ s/^\s+//;
-	    $new_app->{'date'} = $line;
-	    $line=<STDIN>;
-	    next unless defined $line;
-	    
 	    # empty line
 	    while ((defined $line) and ($line =~ /^\s*$/)) {
 		$line=<STDIN>;
 	    }
 	    next unless defined $line;
 
-	    # Category
-	    if ($line !~ /^About: /) {
-		chomp $line;
-		$new_app->{'category'} = $line;
-		while ($line=<STDIN>) {
-		    last if $line =~ /^\s*$/;
-		    chomp $line;
-		    $new_app->{'category'} .= "," . $line;
-		}
-		$new_app->{'category'} = $line unless $line =~ /^\s*$/;
-		$line=<STDIN>;
-		next unless defined $line;
-
-		# empty line
-		while ((defined $line) and ($line =~ /^\s*$/)) {
-		    $line=<STDIN>;
-		}
-		next unless defined $line;
-	    }
-
 	    # about
-	    if ($line =~ /^About: /) {
-		$line =~ s/^About: //;
+	    if ($line =~ /^Description:/) {
+		$line=<STDIN>;
 		$new_app->{'description'} = $line;
 		while ($line=<STDIN>) {
 		    last if $line =~ /^\s*$/;
@@ -818,8 +697,8 @@ sub parse_newsletter()
 	    next unless defined $line;
 
 	    # changes
-	    if ($line =~ /^Changes: /) {
-		$line =~ s/^Changes: //;
+	    if ($line =~ /^Changes:/) {
+		$line=<STDIN>;
 		$new_app->{'changes'} = $line;
 		while ($line=<STDIN>) {
 		    last if $line =~ /^\s*$/;
@@ -834,23 +713,55 @@ sub parse_newsletter()
 	    next unless defined $line;
 
 	    # License
-	    if ($line =~ /^\s*License: /) {
-		$line =~ s/^\s*License: //;
+	    if ($line =~ /^License:/) {
 		chomp $line;
+		$line =~ s/^License: //;
 		$new_app->{'license'} = $line unless $line =~ /^\s*$/;
 		$line=<STDIN>;
 		next unless defined $line;
+
+		# empty line
+		while ((defined $line) and ($line =~ /^\s*$/)) {
+		    $line=<STDIN>;
+		}
+		next unless defined $line;
 	    }
 
-	    # empty line
-	    while ((defined $line) and ($line =~ /^\s*$/)) {
+
+	    # Category
+	    if ($line =~ /^Project Tags:/) {
+		chomp $line;
+		$line =~ s/^Project Tags: //;
+		$new_app->{'category'} = $line;
+		while ($line=<STDIN>) {
+		    last if (($line =~ /^\s*$/) || ($line =~ /Release Tags:/)) ;
+		    chomp $line;
+		    $new_app->{'category'} .= "," . $line;
+		}
+		delete $new_app->{'category'} if $new_app->{'category'} eq '';
 		$line=<STDIN>;
+		next unless defined $line;
+
+		# empty line
+		while ((defined $line) and ($line =~ /^\s*$/)) {
+		    $line=<STDIN>;
+		}
+		next unless defined $line;
+	    }
+
+	    if ($line =~ /^Release Tags:/) {
+		chomp $line;
+		next unless defined $line;
+
+		# empty line
+		while ((defined $line) and ($line =~ /^\s*$/)) {
+		    $line=<STDIN>;
+		}
+		next unless defined $line;
 	    }
-	    next unless defined $line;
 
 	    # URL
-	    if ($line =~ /^\s*URL: /) {
-		$line =~ s/^\s*URL: //;
+	    if ($line =~ /^http:/) {
 		chomp $line;
 		$new_app->{'project_link'} = $line;
 		$line =~ s!/$!!;
@@ -898,20 +809,19 @@ sub parse_newsletter()
 
 	    
 	    # wait for separator  (UGLY, change this routine somehow)
+	    # sorry (much more UGLY than original)
 	    my $end = 0;
 	    while ($end == 0) {
 		chomp $line;
 		if ($line !~ /^\s*$/) {
-		    if (($line =~ tr/- %//c) == 0) {
+		    if ($line =~ /========================================================================/) {
 			$end = 1;
 		    }	
 		}
 		$line=<STDIN>;
 		$end = 1 unless defined $line;
 	    }
-
 	    $new_app = {};
-
 	}
     }
 
@@ -925,7 +835,7 @@ sub parse_newsletter()
 ### send mails
 
     mail_hot_apps($hot_applications, $encoding);
-    mail_new_apps($subject, $articles, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $new_applications, $encoding);
+    mail_new_apps($subject, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $new_applications, $encoding);
 
 }
 
@@ -1024,27 +934,27 @@ EOF
 ##################[ format summary of a "new" mail ]#########################
 
 
-sub get_summary($$$$$$$$)
+sub format_summary($$$$$$$)
 {
-    my ($articles, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $score_killed) = @_;
+    my ($releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $score_killed) = @_;
 
     my $already_seen=@{$skipped_already_seen};
     my $difference=$releases-$releases_new-$already_seen;
-    my $remaining=$releases_new+$articles-$score_killed;
+    my $remaining=$releases_new-$score_killed;
     my $summary = << "EOF";
 	
     This newsletter has been filtered by:
     $id
 
-    It contained $articles articles and $releases releases.
+    It contained $releases releases.
 EOF
     
     if ($releases > 1) {    # 1 release is not enough to ensure proper operation!
 
 	$summary .= << "EOF";
     $already_seen releases have been skipped as 'already seen'.
-    $score_killed articles or releases have been skipped as 'low score'.
-    $remaining articles and releases are shown in this mail,
+    $score_killed releases have been skipped as 'low score'.
+    $remaining releases are shown in this mail,
     while $difference releases have been sent separately as 'hot'.
 
 EOF
@@ -1138,6 +1048,41 @@ sub open_new_mail($$)
 }
 
 
+###########################[ fold a line ]##################################
+
+
+sub fold_line($$$$)
+{
+    my ($string, $max, $separator, $prefix) = (@_);
+    
+    my $ret = ''; 
+
+    while (length $string > $max) {
+        my $pos = $max;
+        while ($pos > 0) {
+            if ( substr($string, $pos, 1) eq $separator ) {
+                $ret .= substr($string, 0, $pos+1) . "\n" . $prefix;
+                $string = substr($string, $pos+1);
+                last;
+            }
+            $pos--;
+        }  
+        if ($pos == 0) {
+          $pos = index($string, $separator);
+          if ( $pos > 0) {
+                $ret .= substr($string, 0, $pos+1) . "\n" . $prefix;
+                $string = substr($string, $pos+1);
+          } else {
+            last;
+            }
+        }
+    }
+     
+    return $ret . $string;
+
+}
+
+
 ###################[ read the configuration file ]###########################
 
 
@@ -1146,6 +1091,7 @@ sub read_config()
     my $config_file = $config->{CONFIGFILE};
     my @scores = ();
     my @catscores = ();
+    my @licscores = ();
 
 ### look for config file
     $config_file =~ s/^~/$ENV{'HOME'}/;
@@ -1203,6 +1149,22 @@ sub read_config()
 			push @{$cfg_warnings}, "SCORE value not numeric at line $.";
 		    }
 		    
+		} elsif ($key eq "LICSCORE") {
+		    
+		    my ($score, $regexp) = split /\t/, $value, 2;
+		    if ((! defined $regexp) or ($regexp eq "")) {
+			warn "$0 warning:\n";
+			warn "no REGEXP given in configuration file at line $.\n";
+			push @{$cfg_warnings}, "no REGEXP given at line $.";
+		    }
+		    elsif ($score =~ /[+-]\d+/) {
+			push @licscores, { 'score' => $score, 'regexp' => $regexp };
+		    } else {
+			warn "$0 warning:\n";
+			warn "SCORE value not numeric in configuration file at line $.\n";
+			push @{$cfg_warnings}, "SCORE value not numeric at line $.";
+		    }
+		    
 		} elsif ( exists $cfg_allowed_keys->{$key} ) {
 		    $config->{$key} = $value;
 		} else {
@@ -1258,6 +1220,75 @@ sub read_config()
 
     $config->{'SCORE'}    = \@scores;
     $config->{'CATSCORE'} = \@catscores;
+    $config->{'LICSCORE'} = \@licscores;
+
+}
+
+
+#####################[ format application entry ]############################
+
+
+sub format_application($)
+{
+    my ($app) = (@_);
+
+    my $text = '';
+
+    if (defined $app->{'subject'}) {
+	$text .= "\n   $app->{'subject'}\n\n";
+    }
+	
+    if (defined $app->{'description'}) {
+	$text .= "$app->{'description'}\n";
+    }
+	
+    if (defined $app->{'changes'}) {
+	$text .= "     changes:";
+	if (defined $app->{'urgency'}) {
+	    $text .= " ($app->{'urgency'} urgency)";
+	}
+	$text .= "\n$app->{'changes'}\n";
+    }
+    
+    if (defined $app->{'author'}) {
+	$text .= "    added by: $app->{'author'}\n";
+    }
+	
+    if (defined $app->{'category'}) {
+	$text .= "    category: " . fold_line($app->{'category'}, 57, ',', '             ') . "\n";
+    }
+	
+    if (defined $app->{'project_link'}) {
+	$text .= "project page: $app->{'project_link'}\n";
+    }
+
+    if (defined $app->{'newslink'}) {
+	$text .= "     details: $app->{'newslink'}\n";
+    }
+	
+    if (defined $app->{'date'}) {
+	$text .= "        date: $app->{'date'}\n";
+    }
+	
+    if (defined $app->{'license'}) {
+	$text .= "     license: $app->{'license'}\n";
+    }
+	
+    if (defined $app->{'project_id'}) {
+	$text .= "  project id: $app->{'project_id'}\n";
+    }
+
+    if (defined $app->{'score'}) {
+	$text .= "       score: $app->{'score'}\n";
+    }
+
+    if (defined $app->{'comments'}) {
+	$text .= "your comment: $app->{'comments'}\n";
+    }                                                                                                                                                                   
+
+    $text .= "\n$separator";
+
+    return $text;
 
 }
 
@@ -1268,75 +1299,20 @@ sub read_config()
 sub mail_hot_apps($$)
 {
     my ($hot_applications, $encoding) = @_;
-    my $new_app;
     my $first_hot = 1;
 
-    while (@{$hot_applications}) {
-	
-	$new_app = pop @{$hot_applications};
-    
+    foreach my $new_app (@{$hot_applications}) {
 	
 	if ($first_hot == 1) {
 	    $first_hot=0;
 	    open_hot_mail($new_app, $encoding);
 	}
-	
-	if (defined $new_app->{'subject'}) {
-	    print MAIL_HOT "\n   $new_app->{'subject'}\n\n";
-	}
-	
-	if (defined $new_app->{'description'}) {
-	    print MAIL_HOT "$new_app->{'description'}\n";
-	}
-	
-	if (defined $new_app->{'changes'}) {
-	    print MAIL_HOT "     changes:";
-	    if (defined $new_app->{'urgency'}) {
-		print MAIL_HOT " ($new_app->{'urgency'} urgency)";
-	    }
-	    print MAIL_HOT "\n$new_app->{'changes'}\n";
-	}
-	
-	if (defined $new_app->{'author'}) {
-	    print MAIL_HOT "    added by: $new_app->{'author'}\n";
-	}
-	
-	if (defined $new_app->{'category'}) {
-	    my @categories = split /,/, $new_app->{'category'};
-	    my $category = shift @categories;
-	    print MAIL_HOT "    category: $category\n";
-	    foreach my $category ( @categories ) {
-		$category =~ s/^\s+// ;
-		print MAIL_HOT "              $category\n";
-	    }
-	}
-	
-	if (defined $new_app->{'project_link'}) {
-	    print MAIL_HOT "project page: $new_app->{'project_link'}\n";
-	}
 
-	if (defined $new_app->{'newslink'}) {
-	    print MAIL_HOT "     details: $new_app->{'newslink'}\n";
-	}
-	
-	if (defined $new_app->{'date'}) {
-	    print MAIL_HOT "        date: $new_app->{'date'}\n";
-	}
-	
-	if (defined $new_app->{'license'}) {
-	    print MAIL_HOT "     license: $new_app->{'license'}\n";
-	}
-	
-	if (defined $new_app->{'project_id'}) {
-	    print MAIL_HOT "  project id: $new_app->{'project_id'}\n";
-	}
-
-	if (defined $new_app->{'comments'}) {
-	    print MAIL_HOT "your comment: $new_app->{'comments'}\n";
-	}
-
-	print MAIL_HOT "\n$separator";
+	# don't show a score on hot entries
+	delete $new_app->{'score'};
 
+	print MAIL_HOT format_application($new_app);
+	
 	if ($config->{'UPDATE_MAIL'} ne "single") {
 	    close_hot();
 	    $first_hot=1;
@@ -1354,7 +1330,7 @@ sub mail_hot_apps($$)
 ###################[ format list of skipped items ]##########################
 
 
-sub get_skipped()
+sub format_skipped()
 {
     my $skipped = "";
 
@@ -1386,7 +1362,7 @@ sub get_skipped()
 
 ########[ format configuration file warnings for "new" mail ]################
 
-sub get_warnings()
+sub format_warnings()
 {
     my $warnings = "";
 
@@ -1410,9 +1386,9 @@ sub get_warnings()
 ######################[ mail all 'new' entries ]#############################
 
 
-sub mail_new_apps($$$$$$$$$$)
+sub mail_new_apps($$$$$$$$$)
 {
-    my ($subject, $articles, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $new_applications, $encoding) = @_;
+    my ($subject, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $new_applications, $encoding) = @_;
     my $new_app;
 
 ### only keep applications with at least minimum score
@@ -1423,19 +1399,19 @@ sub mail_new_apps($$$$$$$$$$)
 
 
 ### sort by score
-    $new_applications = [ sort { $a->{'score'} <=> $b->{'score'} } @{$new_applications} ];
+    $new_applications = [ sort { $b->{'score'} <=> $a->{'score'} } @{$new_applications} ];
 
 
 ### get summary
-    my $summary = get_summary($articles, $releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $score_killed);
+    my $summary = format_summary($releases, $releases_new, $hot_written, $db_new, $db_written, $db_expired, $score_killed);
 
 
 ### get warnings
-    my $warnings = get_warnings();
+    my $warnings = format_warnings();
 
 
 ### get skipped list
-    my $skipped = get_skipped();
+    my $skipped = format_skipped();
 
 
 ### open mailer
@@ -1452,66 +1428,9 @@ sub mail_new_apps($$$$$$$$$$)
     print MAIL_NEW $skipped if $config->{'LIST_SKIPPED'} eq 'top';
 
 
-    while (@{$new_applications}) {
-	
-	$new_app = pop @{$new_applications};
-	
-	if (defined $new_app->{'subject'}) {
-	    print MAIL_NEW "\n   $new_app->{'subject'}\n\n";
-	}
-
-	if (defined $new_app->{'description'}) {
-	    print MAIL_NEW "$new_app->{'description'}\n";
-	}
-
-	if (defined $new_app->{'changes'}) {
-	    print MAIL_NEW "     changes:";
-	    if (defined $new_app->{'urgency'}) {
-		print MAIL_NEW " ($new_app->{'urgency'} urgency)";
-	    }
-	    print MAIL_NEW "\n$new_app->{'changes'}\n";
-	}
-
-	if (defined $new_app->{'author'}) {
-	    print MAIL_NEW "    added by: $new_app->{'author'}\n";
-	}
-
-	if (defined $new_app->{'category'}) {
-	    my @categories = split /,/, $new_app->{'category'};
-	    my $category = shift @categories;
-	    print MAIL_NEW "    category: $category\n";
-	    foreach my $category ( @categories ) {
-		$category =~ s/^\s+// ;
-		print MAIL_NEW "              $category\n";
-	    }
-	}
-	
-	if (defined $new_app->{'project_link'}) {
-	    print MAIL_NEW "project page: $new_app->{'project_link'}\n";
-	}
-
-	if (defined $new_app->{'newslink'}) {
-	    print MAIL_NEW "   news item: $new_app->{'newslink'}\n";
-	}
-
-	if (defined $new_app->{'date'}) {
-	    print MAIL_NEW "        date: $new_app->{'date'}\n";
-	}
-	
-	if (defined $new_app->{'license'}) {
-	    print MAIL_NEW "     license: $new_app->{'license'}\n";
-	}
-	
-	if (defined $new_app->{'project_id'}) {
-	    print MAIL_NEW "  project id: $new_app->{'project_id'}\n";
-	}
-
-	if (defined $new_app->{'score'}) {
-	    print MAIL_NEW "       score: $new_app->{'score'}\n";
-	}
-
-	print MAIL_NEW "\n$separator";
-	     
+### print application entries
+    foreach my $new_app (@{$new_applications}) {
+	print MAIL_NEW format_application($new_app);
     }
     
 ### list skipped items if you want them at the bottom
